﻿/**
* ExpressionStorage v0.0.0 / ScriptUI
* Kareobana(http://atarabi.com/)
*
* dependencies :
*     KIKAKU.Utils 0.2.0
*     KIKAKU.UIBuilder 1.0.0
*/

(function (global) {
  var Utils = KIKAKU.Utils,
  builder = new KIKAKU.UIBuilder(global, 'ExpressionStorage', {version: '0.0.0', author: 'Kareobana', url: 'http://atarabi.com/', filetype: 'json', width: 250});

  builder
  .add('hidden', 'Data', {})
  .add('hidden', 'Refresh', function () {
    var filename = this.get('List');
    if (filename === null) {
      return;
    }
    var data = this.get('Data');
    this.set('Expression', data[filename].expression);
  })
  .add('listbox', 'List', undefined, {
    title: false,
    height: 100,
    callback: function () {
      this.execute('Refresh');
    },
    onDoubleClick: function () {
      this.execute('Apply');
    }
  })
  .add('textarea', 'Expression', '', {
    title: false,
    height: 60,
    callback: function () {
      var filename = this.get('List');
      if (filename === null) {
        return;
      }
      var expression = this.get('Expression'),
      data = this.get('Data');
      data[filename].expression = expression;
      data[filename].updatedAt = new Date();
      this.saveFile(filename, data[filename]);
    }
  })
  .add('script', 'Add', function () {
    var filename = prompt('Input a filename', '');
    if (!filename || filename === '') {
      return;
    }
    var data = this.get('Data');
    if (data[filename]) {
      return alert(filename + ' already exists');
    }

    var expression = this.get('List') === null ? this.get('Expression') : '';
    if (expression === '') {
      var property = Utils.getSelectedProperty();
      if (property !== null && property.canSetExpression && property.expression !== '') {
        expression = property.expression;
      }
    }

    var datum = {
     expression: expression,
     createdAt: new Date(),
     updatedAt: new Date(),
   };
    try {
      this.saveFile(filename, datum);
    } catch (e) {
      return alert(e);
    }
    data[filename] = datum;
    this.addItems('List', filename);
    this.set('List', filename);
    this.execute('Refresh');
  })
  .add('script', 'Delete', function () {
    var filename = this.get('List');
      if (filename === null) {
        return;
      }
      var do_delete = confirm('Delete ' + filename + '?');
      if (do_delete) {
        var data = this.get('Data');
        delete data[filename];
        this.deleteFile(filename);
        this.removeItem('List', filename);
        this.execute('Refresh');
      }
  })
  .add('script', 'Apply', function () {
      var expression = this.get('Expression'),
      properties = Utils.getSelectedProperties();

      if (expression !== '') {
        for (var i = 0, l = properties.length; i < l; i++) {
          var property = properties[i];
          if (property.canSetExpression) {
            property.expression = expression;
          }
        }
      }
  })
  .build(function () {
    var data = this.get('Data'),
    filenames = this.getFilenames();
    for (var i = 0, l = filenames.length; i < l; i++) {
      var filename = filenames[i];
      try {
        data[filename] = this.getFile(filename);
      } catch (e) {
      }
    }
    this.replaceItems('List', filenames);
    this.execute('Refresh');
  });

})(this);
